@@ -4,7 +4,7 @@ import os
 from typing import Dict, Mapping
 
 from dateutil.parser import parse as parse_datestr
-from flask import Blueprint, abort, g, jsonify, make_response, request
+from flask import Blueprint, Response, abort, g, jsonify, make_response, request
 from google.auth import jwt
 
 from trailblazer.server.ext import store
@@ -242,8 +242,44 @@ def post_add_pending_analysis():
             priority=content.get("priority"),
             data_analysis=content.get("data_analysis"),
             ticket_id=content.get("ticket"),
+            workflow_manager=content.get("workflow_manager"),
         )
         data = stringify_timestamps(analysis_obj.to_dict())
         return jsonify(**data), 201
     except Exception as e:
         return jsonify(f"Exception: {e}"), 409
+
+
+@blueprint.route("/set-analysis-uploaded", methods=["PUT"])
+def put_set_analysis_uploaded():
+    content: Response.json = request.json
+
+    try:
+        store.set_analysis_uploaded(
+            case_id=content.get("case_id"), uploaded_at=content.get("uploaded_at")
+        )
+        return jsonify("Success! Uploaded at request sent"), 201
+    except Exception as error:
+        return jsonify(f"Exception: {error}"), 409
+
+
+@blueprint.route("/set-analysis-status", methods=["PUT"])
+def put_set_analysis_status():
+    content: Response.json = request.json
+
+    try:
+        store.set_analysis_status(case_id=content.get("case_id"), status=content.get("status"))
+        return jsonify(f"Success! Analysis set to {content.get('status')} request sent"), 201
+    except Exception as error:
+        return jsonify(f"Exception: {error}"), 409
+
+
+@blueprint.route("/add-comment", methods=["PUT"])
+def put_add_comment():
+    content: Response.json = request.json
+
+    try:
+        store.add_comment(case_id=content.get("case_id"), comment=content.get("comment"))
+        return jsonify("Success! Adding comment request sent"), 201
+    except Exception as error:
+        return jsonify(f"Exception: {error}"), 409
