@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
-"""Store backend in Trailblazer"""
+"""Store backend in Trailblazer."""
 import datetime as dt
 import io
 import logging
 import subprocess
+from pathlib import Path
 from typing import Any, List, Optional
 
 import alchy
@@ -11,27 +11,31 @@ import pandas as pd
 import sqlalchemy as sqa
 from alchy import Query
 from dateutil.parser import parse as parse_datestr
-from ruamel.yaml import safe_load
 
+from trailblazer.apps.tower.api import TowerAPI
 from trailblazer.constants import (
     COMPLETED_STATUS,
     FAILED_STATUS,
     ONGOING_STATUSES,
     SLURM_ACTIVE_CATEGORIES,
     STARTED_STATUSES,
+    TrailblazerStatus,
+    WorkflowManager,
+    FileFormat,
 )
-from trailblazer.exc import EmptySqueueError, TrailblazerError
-from trailblazer.store import models
+from trailblazer.exc import EmptySqueueError, TowerRequirementsError, TrailblazerError
+from trailblazer.io.controller import ReadFile
+from trailblazer.store.models import Model, User, Analysis, Job, Info
 from trailblazer.store.utils import formatters
 
 LOG = logging.getLogger(__name__)
 
 
 class BaseHandler:
-    User = models.User
-    Analysis = models.Analysis
-    Job = models.Job
-    Info = models.Info
+    User = User
+    Analysis = Analysis
+    Job = Job
+    Info = Info
 
     def setup(self):
         self.create_all()
@@ -39,7 +43,7 @@ class BaseHandler:
         new_info = self.Info()
         self.add_commit(new_info)
 
-    def info(self) -> models.Info:
+    def info(self) -> Info:
         """Return metadata entry."""
         return self.Info.query.first()
 
@@ -51,7 +55,7 @@ class BaseHandler:
         metadata.updated_at = dt.datetime.now()
         self.commit()
 
-    def get_analysis(self, case_id: str, started_at: dt.datetime, status: str) -> models.Analysis:
+    def get_analysis(self, case_id: str, started_at: dt.datetime, status: str) -> Analysis:
         """
         used in LOG
         Find a single analysis."""
@@ -118,24 +122,24 @@ class BaseHandler:
             analysis_query = analysis_query.filter_by(is_visible=is_visible)
         if data_analysis:
             analysis_query = analysis_query.filter(
-                models.Analysis.data_analysis.ilike(f"%{data_analysis}%")
+                Analysis.data_analysis.ilike(f"%{data_analysis}%")
             )
         if comment:
-            analysis_query = analysis_query.filter(models.Analysis.comment.ilike(f"%{comment}%"))
+            analysis_query = analysis_query.filter(Analysis.comment.ilike(f"%{comment}%"))
 
         return analysis_query.order_by(self.Analysis.started_at.desc())
 
-    def analysis(self, analysis_id: int) -> Optional[models.Analysis]:
+    def analysis(self, analysis_id: int) -> Optional[Analysis]:
         """
         used by REST
         Get a single analysis by id."""
         return self.Analysis.query.get(analysis_id)
 
-    def get_latest_analysis(self, case_id: str) -> Optional[models.Analysis]:
+    def get_latest_analysis(self, case_id: str) -> Optional[Analysis]:
         return self.analyses(case_id=case_id).first()
 
     def get_latest_analysis_status(self, case_id: str) -> Optional[str]:
-        """Get latest analysis status for a case_id"""
+        """Get the latest analysis status for a case_id"""
         latest_analysis = self.get_latest_analysis(case_id=case_id)
         if latest_analysis:
             return latest_analysis.status
@@ -170,7 +174,8 @@ class BaseHandler:
         email: str = None,
         data_analysis: str = None,
         ticket_id: str = None,
-    ) -> models.Analysis:
+        workflow_manager: str = None,
+    ) -> Analysis:
         """Add pending entry for an analysis."""
         started_at = dt.datetime.now()
         new_log = self.Analysis(
@@ -183,23 +188,24 @@ class BaseHandler:
             priority=priority,
             data_analysis=data_analysis,
             ticket_id=ticket_id,
+            workflow_manager=workflow_manager,
         )
         new_log.user = self.user(email) if email else None
         self.add_commit(new_log)
         return new_log
 
-    def add_user(self, name: str, email: str) -> models.User:
+    def add_user(self, name: str, email: str) -> User:
         """Add a new user to the database."""
         new_user = self.User(name=name, email=email)
         self.add_commit(new_user)
         return new_user
 
-    def archive_user(self, user: models.User, archive: bool = True) -> None:
+    def archive_user(self, user: User, archive: bool = True) -> None:
         """Archive user in the database."""
         user.is_archived = archive
         self.commit()
 
-    def user(self, email: str, include_archived: bool = False) -> models.User:
+    def user(self, email: str, include_archived: bool = False) -> User:
         """Fetch a user from the database."""
         query = self.User.query
 
@@ -248,6 +254,29 @@ class BaseHandler:
         self.commit()
         LOG.info(f"{analysis_obj.family} - status set to COMPLETED")
 
+    def set_analysis_uploaded(self, case_id: str, uploaded_at: dt.datetime) -> None:
+        """Setting analysis uploaded at."""
+        analysis_obj: Analysis = self.get_latest_analysis(case_id=case_id)
+        analysis_obj.uploaded_at: dt.datetime = uploaded_at
+        self.commit()
+
+    def set_analysis_status(self, case_id: str, status: str):
+        """Setting analysis status."""
+        analysis_obj: Analysis = self.get_latest_analysis(case_id=case_id)
+        analysis_obj.status: str = status
+        self.commit()
+        LOG.info(f"{analysis_obj.family} - Status set to {status}")
+
+    def add_comment(self, case_id: str, comment: str):
+        analysis_obj: Analysis = self.get_latest_analysis(case_id=case_id)
+        analysis_obj.comment: str = (
+            " ".join([analysis_obj.comment, comment]) if analysis_obj.comment else comment
+        )
+
+        self.commit()
+
+        LOG.info(f"Adding comment {comment} to analysis {analysis_obj.family}")
+
     def delete_analysis(self, analysis_id: int, force: bool = False) -> None:
         """Delete the analysis output."""
         analysis_obj = self.analysis(analysis_id=analysis_id)
@@ -268,8 +297,10 @@ class BaseHandler:
         job_id_file: Path to slurm id .YAML file as string
         case_id: Unique internal case identifier which is expected to by the only item in the .YAML dict
         ssh : Whether the request is executed from hasta or clinical-db"""
-        job_id_dict = safe_load(open(job_id_file))
-        submitted_job_ids = job_id_dict.get(next(iter(job_id_dict)))
+        job_id: dict = ReadFile.get_content_from_file(
+            file_format=FileFormat.YAML, file_path=Path(job_id_file)
+        )
+        submitted_job_ids = job_id.get(next(iter(job_id)))
         job_ids_string = ",".join(map(str, submitted_job_ids))
         if ssh:
             return (
@@ -317,7 +348,7 @@ class BaseHandler:
             split_timestamp = list("0" * (3 - len(split_timestamp))) + split_timestamp
         return int(
             (parse_datestr(":".join(split_timestamp)) - parse_datestr("0:0:0")).seconds / 60
-            + days * 60
+            + days * 24 * 60
         )
 
     @staticmethod
@@ -348,7 +379,7 @@ class BaseHandler:
         )
         return parsed_df
 
-    def update_jobs(self, analysis_obj: models.Analysis, jobs_dataframe: pd.DataFrame) -> None:
+    def update_slurm_jobs(self, analysis_obj: Analysis, jobs_dataframe: pd.DataFrame) -> None:
         """Parses job dataframe and creates job objects"""
         if len(jobs_dataframe) == 0:
             return
@@ -381,13 +412,13 @@ class BaseHandler:
         for analysis_obj in ongoing_analyses:
             try:
                 self.update_run_status(analysis_id=analysis_obj.id, ssh=ssh)
-            except Exception as e:
+            except Exception as error:
                 LOG.error(
-                    f"Failed to update {analysis_obj.family} - {analysis_obj.id}: {e.__class__.__name__}"
+                    f"Failed to update {analysis_obj.family} - {analysis_obj.id}: {type(error).__name__}"
                 )
 
     @staticmethod
-    def get_elapsed_time(self, analysis_obj: models.Analysis) -> str:
+    def get_elapsed_time(self, analysis_obj: Analysis) -> str:
         """Get elapsed time for the analysis"""
         return str(
             (
@@ -399,58 +430,105 @@ class BaseHandler:
         )
 
     def update_run_status(self, analysis_id: int, ssh: bool = False) -> None:
-        """Query slurm for entries related to given analysis, and update the Trailblazer database"""
-        analysis_obj = self.analysis(analysis_id)
-        if not analysis_obj:
+        """Query entries related to given analysis, and update the Trailblazer database."""
+        analysis: Analysis = self.analysis(analysis_id)
+        if not analysis:
             LOG.warning(f"Analysis {analysis_id} not found!")
             return
+        if analysis.workflow_manager == WorkflowManager.TOWER.value:
+            self.update_tower_run_status(analysis_id=analysis_id)
+        elif analysis.workflow_manager == WorkflowManager.SLURM.value:
+            self.update_slurm_run_status(analysis_id=analysis_id, ssh=ssh)
+
+    def update_slurm_run_status(self, analysis_id: int, ssh: bool = False) -> None:
+        """Query slurm for entries related to given analysis, and update the Trailblazer database"""
+        analysis: Analysis = self.analysis(analysis_id)
         try:
             jobs_dataframe = self.parse_squeue_to_df(
                 squeue_response=self.query_slurm(
-                    job_id_file=analysis_obj.config_path, case_id=analysis_obj.family, ssh=ssh
+                    job_id_file=analysis.config_path, case_id=analysis.family, ssh=ssh
                 ),
                 ssh=ssh,
             )
-            self.update_jobs(analysis_obj=analysis_obj, jobs_dataframe=jobs_dataframe)
+            self.update_slurm_jobs(analysis_obj=analysis, jobs_dataframe=jobs_dataframe)
 
             status_distribution = round(
                 jobs_dataframe.status.value_counts() / len(jobs_dataframe), 2
             )
 
-            LOG.info(f"Status in SLURM: {analysis_obj.family} - {analysis_id}")
+            LOG.info(f"Status in SLURM: {analysis.family} - {analysis_id}")
             LOG.info(jobs_dataframe)
-            analysis_obj.progress = float(status_distribution.get("COMPLETED", 0.0))
+            analysis.progress = float(status_distribution.get("COMPLETED", 0.0))
             if status_distribution.get("FAILED") or status_distribution.get("TIMEOUT"):
                 if status_distribution.get("RUNNING") or status_distribution.get("PENDING"):
-                    analysis_obj.status = "error"
+                    analysis.status = TrailblazerStatus.ERROR.value
                 else:
-                    analysis_obj.status = "failed"
+                    analysis.status = TrailblazerStatus.FAILED.value
 
             elif status_distribution.get("COMPLETED") == 1:
-                analysis_obj.status = "completed"
+                analysis.status = TrailblazerStatus.COMPLETED.value
 
             elif status_distribution.get("PENDING") == 1:
-                analysis_obj.status = "pending"
+                analysis.status = TrailblazerStatus.PENDING.value
 
             elif status_distribution.get("RUNNING"):
-                analysis_obj.status = "running"
+                analysis.status = TrailblazerStatus.RUNNING.value
 
             elif status_distribution.get("CANCELLED") and not (
                 status_distribution.get("RUNNING") or status_distribution.get("PENDING")
             ):
-                analysis_obj.status = "canceled"
+                analysis.status = TrailblazerStatus.CANCELLED.value
 
-            LOG.info(
-                f"Updated status {analysis_obj.family} - {analysis_obj.id}: {analysis_obj.status} "
-            )
+            LOG.info(f"Updated status {analysis.family} - {analysis.id}: {analysis.status} ")
             self.commit()
 
-            analysis_obj.logged_at = dt.datetime.now()
+            analysis.logged_at = dt.datetime.now()
         except Exception as e:
-            LOG.error(f"Error logging case - {analysis_obj.family} : {e.__class__.__name__}")
-            analysis_obj.status = "error"
+            LOG.error(f"Error logging case - {analysis.family} : {e.__class__.__name__}")
+            analysis.status = "error"
             self.commit()
 
+    @staticmethod
+    def query_tower(config_file: str, case_id: str) -> TowerAPI:
+        """Parse a config file to extract a NF Tower workflow ID and return a TowerAPI.
+        Currently only one tower ID is supported."""
+        workflow_id: int = ReadFile.get_content_from_file(
+            file_format=FileFormat.YAML, file_path=Path(config_file)
+        ).get(case_id)[0]
+        tower_api = TowerAPI(workflow_id=workflow_id)
+        if not tower_api.tower_client.meets_requirements:
+            raise TowerRequirementsError
+        return tower_api
+
+    def update_tower_run_status(self, analysis_id: int) -> None:
+        """Query tower for entries related to given analysis, and update the Trailblazer database."""
+        analysis: Analysis = self.analysis(analysis_id)
+        tower_api: TowerAPI = self.query_tower(
+            config_file=analysis.config_path, case_id=analysis.family
+        )
+
+        try:
+            LOG.info(
+                f"Status in Tower: {analysis.family} - {analysis_id} - {tower_api.workflow_id}"
+            )
+            analysis.status: str = tower_api.status
+            analysis.progress: int = tower_api.progress
+            analysis.logged_at: dt.datetime = dt.datetime.now()
+            self.update_jobs(analysis=analysis, jobs=tower_api.get_jobs(analysis_id=analysis.id))
+            self.commit()
+            LOG.info(f"Updated status {analysis.family} - {analysis.id}: {analysis.status} ")
+        except Exception as error:
+            LOG.error(f"Error logging case - {analysis.family} :  {type(error).__name__}")
+            analysis.status: str = TrailblazerStatus.ERROR.value
+            self.commit()
+
+    def update_jobs(self, analysis: Analysis, jobs: List[dict]) -> None:
+        """Updates failed jobs in the analysis."""
+        for job in analysis.failed_jobs:
+            job.delete()
+        analysis.failed_jobs = [self.Job(job) for job in jobs]
+        self.commit()
+
     @staticmethod
     def cancel_slurm_job(slurm_id: int, ssh: bool = False) -> None:
         """Cancel slurm job by slurm job ID"""
@@ -488,4 +566,4 @@ class BaseHandler:
 
 class Store(alchy.Manager, BaseHandler):
     def __init__(self, uri: str):
-        super(Store, self).__init__(config=dict(SQLALCHEMY_DATABASE_URI=uri), Model=models.Model)
+        super(Store, self).__init__(config=dict(SQLALCHEMY_DATABASE_URI=uri), Model=Model)
