@@ -10,6 +10,7 @@ from trailblazer.constants import (
     PRIORITY_OPTIONS,
     STATUS_OPTIONS,
     TYPES,
+    WorkflowManager,
 )
 
 Model = alchy.make_declarative_base(Base=alchy.ModelBase)
@@ -74,6 +75,10 @@ class Analysis(Model):
     progress = Column(types.Float, default=0.0)
     data_analysis = Column(types.String(32))
     ticket_id = Column(types.String(32))
+    uploaded_at = Column(types.DateTime)
+    workflow_manager = Column(
+        types.Enum(*WorkflowManager.list()), default=WorkflowManager.SLURM.value
+    )
 
     failed_jobs = orm.relationship("Job", backref="analysis")
 
